@@ -43,6 +43,7 @@
 #include "ephy-string.h"
 #include "ephy-web-view.h"
 #include "gedit-overlay.h"
+#include "ephy-overlay-escaping-child.h"
 
 #include <errno.h>
 #include <glib/gi18n.h>
@@ -59,6 +60,12 @@ static gboolean ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
 
 #define EPHY_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED, EphyEmbedPrivate))
 
+typedef struct {
+  gchar *text;
+  guint context_id;
+  guint message_id;
+} EphyEmbedStatusbarMsg;
+
 struct _EphyEmbedPrivate
 {
   GtkBox *top_widgets_vbox;
@@ -73,6 +80,14 @@ struct _EphyEmbedPrivate
   guint is_setting_zoom : 1;
   GSList *destroy_on_transition_list;
   GtkWidget *statusbar_label;
+
+  GSList *messages;
+  GSList *keys;
+
+  guint seq_context_id;
+  guint seq_message_id;
+
+  guint tab_message_id;
 };
 
 G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_VBOX)
@@ -275,6 +290,23 @@ ephy_embed_finalize (GObject *object)
                                         ephy_embed_history_cleared_cb,
                                         embed);
 
+  for (list = embed->priv->messages; list; list = list->next) {
+    EphyEmbedStatusbarMsg *msg;
+
+    msg = list->data;
+    g_free (msg->text);
+    g_slice_free (EphyEmbedStatusbarMsg, msg);
+  }
+
+  g_slist_free (embed->priv->messages);
+  embed->priv->messages = NULL;
+
+  for (list = embed->priv->keys; list; list = list->next)
+    g_free (list->data);
+
+  g_slist_free (embed->priv->keys);
+  embed->priv->keys = NULL;
+
   G_OBJECT_CLASS (ephy_embed_parent_class)->finalize (object);
 }
 
@@ -390,9 +422,6 @@ download_requested_cb (WebKitWebView *web_view,
   return TRUE;
 }
 
-/* FIXME: it probably makes sense to move this stuff completely into
- * EphyEmbed now, since it's not an integral part of EphyWebView
- * anymore. */
 void
 _ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label)
 {
@@ -411,6 +440,51 @@ _ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label)
     gtk_widget_show (parent);
 }
 
+static void
+status_message_notify_cb (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed)
+{
+  const char *message;
+  EphyEmbedPrivate *priv;
+
+  message = ephy_web_view_get_status_message (view);
+
+  priv = embed->priv;
+
+  ephy_embed_statusbar_pop (embed, priv->tab_message_id);
+
+  if (message)
+    ephy_embed_statusbar_push (embed, priv->tab_message_id, message);
+}
+
+static void
+window_resize_requested (WebKitWebWindowFeatures *features, GParamSpec *pspec, EphyEmbed *embed)
+{
+  GtkWidget *window;
+  gboolean is_popup;
+  const char *property_name;
+  int width, height;
+
+  window = gtk_widget_get_toplevel (GTK_WIDGET (embed));
+  if (!window || !gtk_widget_is_toplevel (window))
+    return;
+
+  g_object_get (window, "is-popup", &is_popup, NULL);
+  if (!is_popup)
+    return;
+
+  property_name = g_param_spec_get_name (pspec);
+
+  if (g_str_equal (property_name, "x") || g_str_equal (property_name, "y")) {
+    int x, y;
+    g_object_get (features, "x", &x, "y", &y, NULL);
+    gtk_window_move (GTK_WINDOW (window), x, y);
+    return;
+  }
+
+  g_object_get (features, "width", &width, "height", &height, NULL);
+  gtk_window_resize (GTK_WINDOW (window), width, height);
+}
+
 static void
 ephy_embed_constructed (GObject *object)
 {
@@ -419,8 +493,10 @@ ephy_embed_constructed (GObject *object)
   GtkWidget *scrolled_window;
   GtkWidget *paned;
   WebKitWebView *web_view;
+  WebKitWebWindowFeatures *window_features;
   WebKitWebInspector *inspector;
   GtkWidget *overlay;
+  EphyOverlayEscapingChild *escaping_child;
   GtkWidget *frame;
   GtkCssProvider *provider;
   GtkStyleContext *context;
@@ -451,7 +527,10 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_show (frame);
     
   gtk_container_add (GTK_CONTAINER (frame), priv->statusbar_label);
-  gedit_overlay_add (GEDIT_OVERLAY (overlay), frame, GEDIT_OVERLAY_CHILD_POSITION_SOUTH_WEST, 0);
+  escaping_child = ephy_overlay_escaping_child_new (frame);
+  gedit_overlay_add (GEDIT_OVERLAY (overlay),
+                     GTK_WIDGET (escaping_child),
+                     GEDIT_OVERLAY_CHILD_POSITION_SOUTH_WEST, 0);
 
   paned = GTK_WIDGET (embed->priv->paned);
 
@@ -476,6 +555,16 @@ ephy_embed_constructed (GObject *object)
                     "signal::resource-request-starting", G_CALLBACK (resource_request_starting_cb), embed,
                     "signal::download-requested", G_CALLBACK (download_requested_cb), embed,
                     "signal::notify::zoom-level", G_CALLBACK (zoom_changed_cb), embed,
+                    "signal::notify::status-message", G_CALLBACK (status_message_notify_cb), embed,
+                    NULL);
+
+  /* Window features */
+  window_features = webkit_web_view_get_window_features (web_view);
+  g_object_connect (window_features,
+                    "signal::notify::x", G_CALLBACK (window_resize_requested), embed,
+                    "signal::notify::y", G_CALLBACK (window_resize_requested), embed,
+                    "signal::notify::width", G_CALLBACK (window_resize_requested), embed,
+                    "signal::notify::height", G_CALLBACK (window_resize_requested), embed,
                     NULL);
 
   /* The inspector */
@@ -530,6 +619,9 @@ ephy_embed_init (EphyEmbed *embed)
   embed->priv->scrolled_window = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
   embed->priv->paned = GTK_PANED (gtk_vpaned_new ());
   embed->priv->top_widgets_vbox = GTK_BOX (gtk_vbox_new (FALSE, 0));
+  embed->priv->seq_context_id = 1;
+  embed->priv->seq_message_id = 1;
+  embed->priv->tab_message_id = ephy_embed_statusbar_get_context_id (embed, EPHY_EMBED_STATUSBAR_TAB_MESSAGE_CONTEXT_DESCRIPTION);
 
   gtk_scrolled_window_set_policy (embed->priv->scrolled_window,
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -602,3 +694,117 @@ ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
   gtk_container_remove (GTK_CONTAINER (embed->priv->top_widgets_vbox),
                         GTK_WIDGET (widget));
 }
+
+static void
+ephy_embed_statusbar_update (EphyEmbed *embed, const char *text)
+{
+  g_return_if_fail (EPHY_IS_EMBED (embed));
+
+  _ephy_embed_set_statusbar_label (embed, text);
+}
+
+/* Portions of the following code based on GTK+.
+ * License block as follows:
+ *
+ * GTK - The GIMP Toolkit
+ * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ * GtkStatusbar Copyright (C) 1998 Shawn T. Amundson
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * file for a list of people on the GTK+ Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
+ *
+ */
+
+guint
+ephy_embed_statusbar_get_context_id (EphyEmbed *embed, const char  *context_description)
+{
+  char *string;
+  guint id;
+
+  g_return_val_if_fail (EPHY_IS_EMBED (embed), 0);
+  g_return_val_if_fail (context_description != NULL, 0);
+
+  /* we need to preserve namespaces on object datas */
+  string = g_strconcat ("ephy-embed-status-bar-context:", context_description, NULL);
+
+  id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (embed), string));
+  if (id == 0) {
+    EphyEmbedPrivate *priv = embed->priv;
+
+    id = priv->seq_context_id++;
+    g_object_set_data_full (G_OBJECT (embed), string, GUINT_TO_POINTER (id), NULL);
+    priv->keys = g_slist_prepend (priv->keys, string);
+  } else
+    g_free (string);
+
+  return id;
+}
+
+guint
+ephy_embed_statusbar_push (EphyEmbed *embed, guint context_id, const char *text)
+{
+  EphyEmbedPrivate *priv;
+  EphyEmbedStatusbarMsg *msg;
+
+  g_return_val_if_fail (EPHY_IS_EMBED (embed), 0);
+  g_return_val_if_fail (context_id != 0, 0);
+  g_return_val_if_fail (text != NULL, 0);
+
+  priv = embed->priv;
+
+  msg = g_slice_new (EphyEmbedStatusbarMsg);
+  msg->text = g_strdup (text);
+  msg->context_id = context_id;
+  msg->message_id = priv->seq_message_id++;
+
+  priv->messages = g_slist_prepend (priv->messages, msg);
+
+  ephy_embed_statusbar_update (embed, text);
+
+  return msg->message_id;
+}
+
+void
+ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
+{
+  EphyEmbedPrivate *priv;
+  EphyEmbedStatusbarMsg *msg;
+  GSList *list;
+
+  g_return_if_fail (EPHY_IS_EMBED (embed));
+  g_return_if_fail (context_id != 0);
+
+  priv = embed->priv;
+
+  for (list = priv->messages; list; list = list->next) {
+    EphyEmbedStatusbarMsg *msg = list->data;
+
+    if (msg->context_id == context_id) {
+      priv->messages = g_slist_remove_link (priv->messages, list);
+      g_free (msg->text);
+      g_slice_free (EphyEmbedStatusbarMsg, msg);
+      g_slist_free_1 (list);
+      break;
+    }
+  }
+
+  msg = priv->messages ? priv->messages->data : NULL;
+  ephy_embed_statusbar_update (embed, msg ? msg->text : NULL);
+}
