@@ -15,7 +15,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #ifndef __EPHY_TREE_MODEL_NODE_H
@@ -36,20 +35,22 @@ G_BEGIN_DECLS
 
 typedef void (*EphyTreeModelNodeValueFunc) (EphyNode *node, GValue *value, gpointer user_data);
 
+typedef struct _EphyTreeModelNode EphyTreeModelNode;
+typedef struct _EphyTreeModelNodeClass EphyTreeModelNodeClass;
 typedef struct _EphyTreeModelNodePrivate EphyTreeModelNodePrivate;
 
-typedef struct
+struct _EphyTreeModelNode
 {
 	GObject parent;
 
 	/*< private >*/
 	EphyTreeModelNodePrivate *priv;
-} EphyTreeModelNode;
+};
 
-typedef struct
+struct _EphyTreeModelNodeClass
 {
 	GObjectClass parent;
-} EphyTreeModelNodeClass;
+};
 
 GType              ephy_tree_model_node_get_type         (void);
 
@@ -73,4 +74,8 @@ void               ephy_tree_model_node_iter_from_node   (EphyTreeModelNode *mod
 
 G_END_DECLS
 
+#if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
+#error "Only <epiphany/epiphany.h> can be included directly."
+#endif
+
 #endif /* EPHY_TREE_MODEL_NODE_H */
