@@ -15,7 +15,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
@@ -30,8 +29,7 @@
 #include "ephy-node-common.h"
 #include "ephy-debug.h"
 
-#include <gtk/gtkaction.h>
-#include <gtk/gtkactiongroup.h>
+#include <gtk/gtk.h>
 #include <string.h>
 
 static void
