@@ -23,9 +23,9 @@
 #define __EPHY_EPIPHANY_H_INSIDE__
 
 /* From GTK_CHECK_VERSION */
-#define EPHY_MAJOR_VERSION (2)
-#define EPHY_MINOR_VERSION (91)
-#define EPHY_MICRO_VERSION (5)
+#define EPHY_MAJOR_VERSION (3)
+#define EPHY_MINOR_VERSION (0)
+#define EPHY_MICRO_VERSION (1)
 
 #define EPHY_CHECK_VERSION(major, minor, micro)\
   (EPHY_MAJOR_VERSION > (major) || \
@@ -38,10 +38,10 @@
 #include <epiphany/ephy-bookmarks.h>
 #include <epiphany/ephy-bookmarks-type-builtins.h>
 #include <epiphany/ephy-dialog.h>
+#include <epiphany/ephy-download.h>
 #include <epiphany/ephy-embed-container.h>
 #include <epiphany/ephy-embed-event.h>
 #include <epiphany/ephy-embed.h>
-#include <epiphany/ephy-embed-persist.h>
 #include <epiphany/ephy-embed-prefs.h>
 #include <epiphany/ephy-embed-shell.h>
 #include <epiphany/ephy-embed-single.h>
