@@ -78,7 +78,8 @@ popup_cmd_link_in_new_tab (GtkAction *action,
 	ephy_shell_new_tab (ephy_shell, window, embed,
 			    g_value_get_string (&value),
 			    EPHY_NEW_TAB_OPEN_PAGE |
-			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+			    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
+			    EPHY_NEW_TAB_APPEND_AFTER);
 	g_value_unset (&value);
 }
 
@@ -221,7 +222,6 @@ save_property_url (GtkAction *action,
 
 	download = ephy_download_new_for_uri (location);
 	ephy_download_set_window (download, GTK_WIDGET (window));
-	g_value_unset (&value);
 
 	if (ask_dest)
 	{
@@ -249,6 +249,8 @@ save_property_url (GtkAction *action,
 		ephy_download_set_auto_destination (download);
 		ephy_download_start (download);
 	}
+
+	g_value_unset (&value);
 }
 
 void
