@@ -1008,9 +1008,6 @@ static const PrefsDialogPreference preferences[] =
 	{ "use_gnome_fonts_checkbutton", "active",
 	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_USE_GNOME_FONTS,
 	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "min_size_spinbutton", "value",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_FONT_MIN_SIZE,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
 	{ "popups_allow_checkbutton", "active",
 	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_POPUPS,
 	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
